# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.http import response
from django.urls import reverse
from django.urls.resolvers import get_ns_resolver

from .base import WeChatTestCase


class UtilDecoratorTestCase(WeChatTestCase):
    pass
