# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from .base import WeChatTestCase


class AdminSiteTestCase(WeChatTestCase):
    def test_admin_view(self):
        """测试admin view"""
        # 测试request能正确拿到appid
        pass

        # 测试响应有extra_context
        pass

