# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from .base import WeChatTestCase


class SettingTestCase(WeChatTestCase):
    def test_settings(self):
        """测试配置是否有效"""
        pass
