{% extends 'admin/wechat_django/change_form.html' %}
{% load static %}
{% block admin_change_form_document_ready %}
{{ block.super }}
<script type="text/javascript" src="{% static 'wechat_django/admin/menuform.js' %}"></script>
{% endblock %}