{% extends 'admin/object_tool/baseview.html' %}
{% load i18n admin_urls static admin_list admin_modify %}

{% block breadcrumbs %}
{% include 'admin/wechat_django/breadcrumbs.html' with view="list_view" %}
{% endblock %}

{% block object-tools-items %}
  {% include 'admin/object_tool/object-tools-items.html' %}
  {% if has_add_permission %}
  <li>
    {% url cl.opts|admin_urlname:'add' wechat_app_id=wechat_app_id as add_url %}
    <a href="{% add_preserved_filters add_url is_popup to_field %}" class="addlink">
      {% blocktrans with cl.opts.verbose_name as name %}Add {{ name }}{% endblocktrans %}
    </a>
  </li>
  {% endif %}
{% endblock %}

{% block result_list %}
{% if action_form and actions_on_top and not cl.show_admin_actions %}{% admin_actions %}{% endif %}
{{ block.super }}
{% endblock %}

{% block search %}{% search_form cl %}{% endblock %}