# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2019-05-25 08:26
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wechat_django', '0004_templatemessage'),
    ]

    operations = [
        migrations.AddField(
            model_name='material',
            name='alias',
            field=models.CharField(blank=True, help_text='\u7d20\u6750\u522b\u540d,\u7528\u4e8e\u7a0b\u5e8f\u5feb\u901f\u67e5\u8be2\u7d20\u6750,\u5355app\u4e0b\u552f\u4e00', max_length=16, null=True, verbose_name='alias'),
        ),
        migrations.AddField(
            model_name='template',
            name='alias',
            field=models.CharField(blank=True, help_text='\u6a21\u677f\u522b\u540d,\u7528\u4e8e\u7a0b\u5e8f\u5feb\u901f\u67e5\u8be2\u6a21\u677f,\u5355app\u4e0b\u552f\u4e00', max_length=16, null=True, verbose_name='alias'),
        ),
        migrations.AddField(
            model_name='wechatuser',
            name='alias',
            field=models.CharField(blank=True, help_text='\u7528\u6237\u522b\u540d,\u7528\u4e8e\u7a0b\u5e8f\u5feb\u901f\u67e5\u8be2\u7528\u6237,\u5355app\u4e0b\u552f\u4e00', max_length=16, null=True, verbose_name='alias'),
        ),
        migrations.AlterField(
            model_name='wechatapp',
            name='type',
            field=models.PositiveSmallIntegerField(choices=[(4, 'MINIPROGRAM'), (0, 'OTHER'), (1, 'SERVICEAPP'), (2, 'SUBSCRIBEAPP')], default=1, verbose_name='type'),
        ),
        migrations.AlterUniqueTogether(
            name='material',
            unique_together=set([('app', 'media_id'), ('app', 'alias')]),
        ),
        migrations.AlterUniqueTogether(
            name='template',
            unique_together=set([('app', 'template_id'), ('app', 'alias')]),
        ),
        migrations.AlterUniqueTogether(
            name='wechatuser',
            unique_together=set([('unionid', 'app'), ('app', 'openid'), ('app', 'alias')]),
        ),
        migrations.AlterField(
            model_name='wechatuser',
            name='nickname',
            field=models.CharField(max_length=32, null=True, verbose_name='nickname'),
        ),
    ]
