# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2019-05-01 08:52
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('wechat_django', '0002_usertags'),
    ]

    operations = [
        migrations.CreateModel(
            name='Session',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('type', models.PositiveSmallIntegerField()),
                ('auth', jsonfield.fields.JSONField(default={})),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='updated at')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='sessions', to='wechat_django.WeChatUser')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AlterModelOptions(
            name='usertag',
            options={'ordering': ('app', 'id'), 'verbose_name': 'user tag', 'verbose_name_plural': 'user tags'},
        ),
        migrations.AlterField(
            model_name='wechatapp',
            name='type',
            field=models.PositiveSmallIntegerField(choices=[(4, 'MINIPROGRAM'), (1, 'SERVICEAPP'), (2, 'SUBSCRIBEAPP')], default=1, verbose_name='type'),
        ),
        migrations.AlterUniqueTogether(
            name='usertag',
            unique_together=set([('app', 'id')]),
        ),
        migrations.AlterIndexTogether(
            name='usertag',
            index_together=set([('app', 'name')]),
        ),
    ]
