# -*- coding: utf-8 -*-
# flake8: noqa
from __future__ import unicode_literals

from .wechat import default_site as wechat, WeChatSite
from .admin import default_site as admin, WeChatAdminSite, WeChatAdminSiteMixin
