{% extends 'admin/delete_confirmation.html' %}
{% load i18n admin_urls %}

{% block breadcrumbs %}
{% include 'admin/wechat_django/breadcrumbs.html' with view_short_description="Delete" %}
{% endblock %}